--[[
********************************
Licence: GPL
(C)2012 rudi, Version: 0.4

NMEA:GPS
********************************
@title nmea gps V0.4
@param p Protokoll
@values p Aus Ein
@default p 0
]]

--[[
Logfile
]]
function get_mode_str()
    if get_mode() then
        return "REC "
    else
        return "PLAY"
    end
end

function logNMEA(status, fn, tab)
    if status == 1 and fn and tab then
        if not hlog then
            local log_name = "A/CHDK/LOGS/"..fn..".log"
            if os.stat(log_name) then
                hlog=io.open(log_name,"ab")
            else
                hlog=io.open(log_name,"wb")
            end
       end
       if tab then
           hlog:write(string.format("* Mode: %s, Date: %s", get_mode_str(), os.date("%d.%m.%Y %X")).."\n")
           for i = 1, #tab do hlog:write(tab[i].."\n") end
       end
    elseif hlog then
        hlog:close()
        hlog = nil
    end
end

--[[
Zeichenfunktionen
]]
function init_draw()
    require("drawings")
    drawId = {}
    tLevelSize = {-1, -1, -1, -1, -1, -1, -1}
    c_bg = "grey_dark"
    c_trans = "trans"
    c_txt = "string"
    c_wnd = {}
    c_wnd.left = 20
    c_wnd.top = 20
    c_wnd.width = 320
    c_wnd.height = 180
    c_wnd.right = c_wnd.left + c_wnd.width
    c_wnd.bottom = c_wnd.top + c_wnd.height
    c_lineHeight = 20
    c_lineLeft = 25
    c_lineId = {"lat", "long", "alt", "speed", "track", "date", "time"}
    c_lineFn = {function () return nmea.getLatitudeStr(tGps) end,
                function () return nmea.getLongitudeStr(tGps) end,
                function () return nmea.getAltitudeStr(tGps) end,
                function () return nmea.getSpeedStr(tGps) end,
                function () return nmea.getTrackStr(tGps) end,
                function () return nmea.getDateStampStr(tGps) end,
                function () return nmea.getTimeStampStr(tGps) end}
    c_satLevelWidth = 120
    --init text IDs
    drawId.mainwindow = draw.add("rectf", c_wnd.left-3, c_wnd.top-3, c_wnd.right+3, c_wnd.bottom+3, "grey", c_bg, 2)
    drawId.titleBG = draw.add("rectf", c_wnd.left, c_wnd.top, c_wnd.right, c_wnd.top+c_lineHeight, c_trans, "white")
    drawId.titleST = draw.add(c_txt, c_wnd.left+20, c_wnd.top+2, title_main, "black", "white")
    for i, cell in ipairs(c_lineId) do
        drawId[cell] = draw.add(c_txt, c_wnd.left, c_wnd.top,"", c_bg, c_bg)
    end
    --init sat nr und level indicator IDs
    for i = 1, 7 do
        drawId["satNr"..tonumber(i)] = draw.add(c_txt, c_wnd.right-20, c_wnd.top+i*c_lineHeight+12, "",  c_bg,  c_bg)
        drawId["satLevBg"..tonumber(i)] = draw.add("rect", c_wnd.right-c_satLevelWidth-30-2, c_wnd.top+i*c_lineHeight+13-2, c_wnd.right-30-2, c_wnd.top+i*c_lineHeight+13+11-2, c_bg)
        drawId["satLev"..tonumber(i)] = draw.add("rectf", c_wnd.right-c_satLevelWidth-30, c_wnd.top+i*c_lineHeight+13, c_wnd.right-30, c_wnd.top+i*c_lineHeight+13+11, c_bg, c_bg, 0)
    end
    draw.redraw()
end

function exit_draw()
    draw.clear()
end

function draw_title(title, bg_color)
    draw.replace(drawId.titleBG, "rectf", c_wnd.left, c_wnd.top, c_wnd.right, c_wnd.top + c_lineHeight, "trans", bg_color)
    draw.replace(drawId.titleST, c_txt, c_wnd.left + 20, c_wnd.top + 2, title_main..title, "black", bg_color)
end

function draw_GPSvalues(color)
    for i, cell in ipairs(c_lineId) do
        draw.replace(drawId[cell], c_txt, c_wnd.left + c_lineLeft, c_wnd.top + i*c_lineHeight + 12, c_lineFn[i](), color , c_bg)
    end
end

function draw_off()
    if lastFixStatus ~= FixStatus then
        draw_title("- ausgeschaltet", "white")
        draw_GPSvalues(c_bg)
        draw_sat({}, FixStatus)
        fdraw = true
     end
end

function draw_noData()
    if lastFixStatus ~= FixStatus then
        draw_title("- suche Satelliten", "grey")
        draw_GPSvalues("grey")
        fdraw = true
    end
    fdraw = draw_sat(tSat, FixStatus) or fdraw
end

function draw_fix()
    if lastFixStatus ~= FixStatus then
        local fixStr, fixCol
        if FixStatus == 2 then
            fixStr = "- 2D"
            fixCol = "yellow"
        elseif FixStatus == 3 then
            fixStr = "- 3D"
            fixCol = "green"
        elseif FixStatus == 6 then
            fixStr = "- geschtzt"
            fixCol = "red_dark"
        else
            fixStr = "- erfasse Daten"
            fixCol = "red"
        end
        draw_title(fixStr, fixCol)
        fdraw = true
    end
    fdraw = draw_sat(tSat, FixStatus) or fdraw
    if FixStatus > 0 then
        draw_GPSvalues("white")
        fdraw = true
    end
end

-- tinfo: [1]nr [2]level
function draw_satNr(pos, tinfo, color)
    local nr = "--"
    if tinfo and (tinfo[1] > 0) then nr = string.format("%2d", tinfo[1]) end
    draw.replace(drawId["satNr"..pos], c_txt, c_wnd.right - 20, c_wnd.top + pos*c_lineHeight + 12, nr, color, c_bg)
end

function draw_satLevel(pos, tinfo, color, colorBg)
    local res = false
    if tinfo[2] then
        local size = tinfo[2] * c_satLevelWidth / 100
        if (size ~= tLevelSize[pos]) or (lastFixStatus ~= FixStatus) then
            tLevelSize[pos] = size
            if tinfo[2] == 0 then color = c_bg end
            res = true
            draw.replace(drawId["satLevBg"..pos], "rect", c_wnd.right - c_satLevelWidth - 30 - 2, c_wnd.top + pos*c_lineHeight + 13 - 2, c_wnd.right - 30 + 2, c_wnd.top + pos*c_lineHeight + 13 + 11 + 2, colorBg)
            draw.replace(drawId["satLev"..pos], "rectf", c_wnd.right - 30 - size, c_wnd.top + pos*c_lineHeight + 13, c_wnd.right - 30, c_wnd.top + pos*c_lineHeight + 13 + 11, c_bg, color, 1)
        end
    end
    return res
end

function draw_sat(tsat, status)
    local res = false
    local colorNr = "grey"
    local colorLev = "grey"
    if status > 0 then
        colorNr = "white"
        colorLev = "blue_dark"
    end
    if tsat then
        for i = 1, 7 do
            if tsat and (type(tsat[i]) == "table") then
                draw_satNr(i, tsat[i], colorNr)
                res = draw_satLevel(i, tsat[i], colorLev, colorNr) or res 
            else
                draw_satNr(i, {i, 0}, c_bg)
                res = draw_satLevel(i, {i, 0}, c_bg, c_bg) or res
            end
        end
    end
    return res
end

--[[
Programmfunktionen
]]
function getFixStatus(tgps)
    ---- -1=GPS off, 0=no data, 1=Fix, 2=2D, 3=3D, 6=estimated
    if not tgps or  not nmea.getGpsPower() then return -1
    elseif not tgps.Status or not tgps.FixQuality then return 0
    elseif tgps.MeasureMode and tonumber(tgps.MeasureMode) then
        if tgps.FixQuality and (tgps.FixQuality == "6") then
            return tonumber(tgps.FixQuality)
        else
            return tonumber(tgps.MeasureMode)
        end
    end
    return 0
end

function getSatStatus(tgps)
    local t = {}
    if tgps.SatTrackList then
        for i = 1, #tgps.SatTrackList do
            t[#t+1] = {tgps.SatTrackList[i], 0}
            if #t == 7 then break end
        end
    end
    if tgps.SatInfo then
        if #t == 0 then
            for i = 1, #tgps.SatInfo do
                t[#t+1] = {tgps.SatInfo[i][1], tgps.SatInfo[i][4]}
                if #t == 7 then break end
            end
        else
            for i = 1, #t do
                if tgps.SatInfo[i] and tgps.SatInfo[i][4] then
                    t[i][2] = tgps.SatInfo[i][4]
                end
            end
        end
    end
    return t
end

function getKey()
    --if is_pressed("right") then return 1 end
    --if is_pressed("left") then return 2 end
    if is_pressed("set") then return 3 end
    return 0
end

function run()
    tNmea = nil
    tGps = nil
    tSat = {}
    lastFixStatus = -2
    fdraw = false
    title_main = "NMEA:GPS "
    local lastKey = 0

    cls()
    set_console_layout(3, 0, 42, 1)
    init_draw()
    sleep(200)

    nmea.setGpsPower(true)
    print("Ende='SET'")

    while lastKey ~= 3 do

        tNmea = nmea.getData()
        logNMEA(p, "NMEA_GPS", tNmea)
        tGps = nmea.getGpsTable(tNmea)
        FixStatus = getFixStatus(tGps)

        if tGps then
            tSat = getSatStatus(tGps)
        end

        fdraw = false
        if FixStatus < 0 then
            draw_off()
        elseif FixStatus == 0 then
            draw_noData()
        else
            draw_fix()
        end
        if fdraw then draw.overdraw() end

        lastKey = getKey()
        if lastKey ~= 3 then
            wait_click(1000 - get_tick_count()%1000)
            if lastKey == 0 then lastKey = getKey() end
        end
    end
end

function restore()
    print("beende ...")
    nmea.setGpsPower(lastGpsPower)
    logNMEA(0)

    exit_draw()
    cls()
    set_console_layout(0, 0, 25, 5)
end

-----MAIN-----
hlog = nil

nmea = require("nmeaLib")
if type(nmea) == "table" then
    lastGpsPower = nmea.getGpsPower()
    run()
    restore()
elseif nmea == 1 then
    print("OPT_LUA_CALL_NATIVE wird bentigt!")
elseif nmea == -1 then
    print("Nicht untersttzte Kamera!")
else
    print("Unbekannter Fehler!")
end
